package serialTest;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;

import gnu.io.CommPort;
import gnu.io.CommPortIdentifier;
import gnu.io.SerialPort;
 
public class TwoWaySerialComm {
 
  void connect( String portName ) throws Exception {
    CommPortIdentifier portIdentifier = CommPortIdentifier
        .getPortIdentifier( portName );
    if( portIdentifier.isCurrentlyOwned() ) {
      System.out.println( "Error: Port is currently in use" );
    } else {
      int timeout = 2000;
      CommPort commPort = portIdentifier.open( this.getClass().getName(), timeout );
 
      if( commPort instanceof SerialPort ) {
        SerialPort serialPort = ( SerialPort )commPort;
        serialPort.setSerialPortParams( 115200,
                                        SerialPort.DATABITS_8,
                                        SerialPort.STOPBITS_1,
                                        SerialPort.PARITY_NONE );
 
        InputStream in = serialPort.getInputStream();
        OutputStream out = serialPort.getOutputStream();
 
        ( new Thread( new SerialReader( in ) ) ).start();
        ( new Thread( new SerialWriter( out ) ) ).start();
 
      } else {
        System.out.println( "Error: Only serial ports are handled by this example." );
      }
    }
  }
 
  public static class SerialReader implements Runnable {
 
    InputStream in;
 
    public SerialReader( InputStream in ) {
      this.in = in;
    }
 
    public void run() {
      byte[] buffer = new byte[ 1024 ];
      int len = -1;
      try {
        while( ( len = this.in.read( buffer ) ) > -1 ) {
          System.out.print( new String( buffer, 0, len ) );
        }
      } catch( IOException e ) {
        e.printStackTrace();
      }
    }
  }
 
  public static class SerialWriter implements Runnable {
 
    OutputStream out;
 
    public SerialWriter( OutputStream out ) {
      this.out = out;
    }
 
    public void run() {
      try {
       String test = "t";
    	  int c = 116;
        while( ( c = System.in.read() ) > -1 ) {
          //this.out.write( c );
        	this.out.write('t');
        	this.out.flush();
        }
      } catch( IOException e ) {
        e.printStackTrace();
      }
    }
  }
  
  static void listPorts()
  {
	  System.out.println("listPorts()");
      java.util.Enumeration<CommPortIdentifier> portEnum = CommPortIdentifier.getPortIdentifiers();
      while ( portEnum.hasMoreElements() ) 
      {
          CommPortIdentifier portIdentifier = portEnum.nextElement();
          System.out.println(portIdentifier.getName()  +  " - " +  getPortTypeName(portIdentifier.getPortType()) );
      }        
  }
  
  static String getPortTypeName ( int portType )
  {
      switch ( portType )
      {
          case CommPortIdentifier.PORT_I2C:
              return "I2C";
          case CommPortIdentifier.PORT_PARALLEL:
              return "Parallel";
          case CommPortIdentifier.PORT_RAW:
              return "Raw";
          case CommPortIdentifier.PORT_RS485:
              return "RS485";
          case CommPortIdentifier.PORT_SERIAL:
              return "Serial";
          default:
              return "unknown type";
      }
  }

 
  public static void main( String[] args ) {
	  
	 // listPorts();
	  
    try {
      ( new TwoWaySerialComm() ).connect( "/dev/ttyACM0" );
    } catch( Exception e ) {
      e.printStackTrace();
    }
  }
}